#
# spec file for package nbd
#
# Copyright (c) 2015 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


Name:           nbd
Version:        3.11
Release:        4.5
Summary:        Network Block Device Server and Client Utilities
License:        GPL-2.0+
Group:          Productivity/Networking/Other
Url:            http://nbd.sourceforge.net/
Source0:        http://downloads.sourceforge.net/project/%{name}/%{name}/%{version}/%{name}-%{version}.tar.xz
Source1:        %{name}-server.service
Source2:        init.nbd-server
Source3:        config.example
Source4:        nbd-server.sysconfig
Patch2:         nbd-2.9.25-close.diff
Patch3:         nbd-2.9.25-doxyfile.diff
# fix return value
Patch4:         no-return-nonvoid.patch
BuildRequires:  doxygen
BuildRequires:  glib2-devel >= 2.26.0
BuildRequires:  xz
Requires(pre):  %fillup_prereq
Requires(pre):  coreutils
Suggests:       nbd-doc
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
%if 0%{?suse_version} >= 1210
BuildRequires:  systemd-rpm-macros
%systemd_requires
%else
Requires(pre):  %insserv_prereq
%endif

%description
This package contains nbd-server. It is the server backend for the nbd
network block device driver that's in the Linux kernel.

nbd can be used to have a filesystem stored on another machine. It does
provide a block device, not a file system; so unless you put a
clustering filesystem on top of it, you can't access it simultaneously
from more than one client. Use NFS or a real cluster FS (such as
ocfs2) if you want to do this. nbd-server can export a file (which may
contain a filesystem image) or a partition. Swapping over nbd is
possible as well, though it's said not to be safe against OOM and
should not be used for that case. nbd-server also has a copy-on-write
mode where changes are saved to a separate file and thrown away when
the connection closes.

The package also contains the nbd-client tools, which you need to
configure the nbd devices on the client side.

%package doc
Summary:        Network Block Device Server and Client Utilities
Group:          Productivity/Networking/Other
Requires:       nbd = %{version}

%description doc
This package contains the HTML documentation for the network block
device (nbd) utilities.

nbd can be used to have a filesystem stored on another machine. It does
provide a block device, not a file system; so unless you put a
clustering filesystem on top of it, you can't access it simultaneously
from more than one client. Use NFS or a real cluster FS (such as
ocfs2) if you want to do this. nbd-server can export a file (which may
contain a filesystem image) or a partition. Swapping over nbd is
possible as well, though it's said not to be safe against OOM and
should not be used for that case. nbd-server also has a copy-on-write
mode where changes are saved to a separate file and thrown away when
the connection closes.

%prep
%setup -q
%patch2 -p1
%patch3 -p1
%patch4 -p1

%build
export CFLAGS="%{optflags} -fstack-protector -fno-strict-aliasing"
%configure

make %{?_smp_mflags}
doxygen doc/Doxyfile.in

%install
make install DESTDIR=%{buildroot} man_MANS='nbd-client.8 nbd-server.1 nbd-server.5 nbd-trdump.1'
mkdir -p %{buildroot}%{_sysconfdir}/init.d
mkdir -p %{buildroot}%{_prefix}/bin

%if 0%{?suse_version} >= 1210
install -D -m 0644  %{SOURCE1} %{buildroot}/%{_unitdir}/%{name}-server.service
ln -s  	%{_sbindir}/service %{buildroot}%{_sbindir}/rc%{name}-server
%else
install %{SOURCE2} %{buildroot}%{_initddir}/nbd-server
ln -s ../..%{_initddir}/nbd-server %{buildroot}%{_sbindir}/rc%{name}-server
%endif

#echo "#Port	file	options" > $RPM_BUILD_ROOT/etc/nbd-server.conf
mkdir -p %{buildroot}%{_sysconfdir}/nbd-server
touch %{buildroot}%{_sysconfdir}/nbd-server/config
touch %{buildroot}%{_sysconfdir}/nbd-server/allow
install -D -m 0644 %{SOURCE3} %{buildroot}%{_sysconfdir}/nbd-server/config.example
install -D -m 0644 %{SOURCE4} %{buildroot}%{_localstatedir}/adm/fillup-templates/sysconfig.%{name}-server

%files
%defattr(-,root,root)
%attr(0755,root,root) %{_sbindir}/nbd-client
%attr(0755,root,root) %{_bindir}/nbd-server
%attr(0755,root,root) %{_bindir}/nbd-trdump
%{_sbindir}/rcnbd-server
%if 0%{?suse_version} >= 1210
%{_unitdir}/%{name}-server.service
%else
%{_initddir}/%{name}-server
%endif
%{_mandir}/man1/nbd-server.1.gz
%{_mandir}/man1/nbd-trdump.1.gz
%{_mandir}/man5/nbd-server.5.gz
%{_mandir}/man8/nbd-client.8.gz
%doc README.md
%dir %{_sysconfdir}/nbd-server
%ghost %config(noreplace) %{_sysconfdir}/nbd-server/config
%ghost %config(noreplace) %{_sysconfdir}/nbd-server/allow
%config %{_sysconfdir}/nbd-server/config.example
%{_localstatedir}/adm/fillup-templates/sysconfig.%{name}-server

%files doc
%defattr(-,root,root)
%doc doc/html

%post
%if 0%{?suse_version} >= 1210
%service_add_post %{name}-server.service
%fillup_only -n nbd-server
%else
%{fillup_and_insserv -f nbd-server}
%endif
if test -e %{_sysconfdir}/nbd-server.conf; then
  # Do we have to create a generic section?
  unset generic
  if test -e %{_sysconfdir}/nbd-server/config; then generic=1; fi
  grep -vE '^(#|[[:blank:]]*$)' %{_sysconfdir}/nbd-server.conf |
  while read port file opts; do
    if test -z "$generic"; then
      echo > %{_sysconfdir}/nbd-server/config
      echo "[generic]" >> %{_sysconfdir}/nbd-server/config
      echo "	# No generic options yet" >> %{_sysconfdir}/nbd-server/config
      echo >> %{_sysconfdir}/nbd-server/config
      generic=1
    fi
    FN=${file%/*}
    nm="cvt.$port.${FN##*/}.${file##*/}"
    echo " ... convert $port $file $opts -> $nm"
    %{_bindir}/nbd-server $port $file $opts -o "$nm" >> %{_sysconfdir}/nbd-server/config
  done
  mv %{_sysconfdir}/nbd-server.conf %{_sysconfdir}/nbd-server.conf.converted
fi

%postun
%if 0%{?suse_version} >= 1210
%service_del_postun %{name}-server.service
%else
%insserv_cleanup
%restart_on_update nbd-server
%endif

%pre
%if 0%{?suse_version} >= 1210
%service_add_pre %{name}-server.service
%endif

%preun
%if 0%{?suse_version} >= 1210
%service_del_preun %{name}-server.service
%else
%{stop_on_removal nbd-server}
%endif

%changelog
* Thu May 28 2015 mpluskal@suse.com
- Update to 3.11
  * nbd-server: fix unsafe signal handling
  * define error values as part of the protocol
  * docs/proto: clarify NBD_CMD_FLUSH
- Drop nbd_signaling_CVE-2015-0847.patch as fix is included in
  upstream
* Mon May 11 2015 mpluskal@suse.com
- Fix CVE-2015-0847
  * nbd_signaling_CVE-2015-0847.patch
* Fri Apr 17 2015 mpluskal@suse.com
- Update to 3.10
  * There really is only one change since 3.9: drop of the
    "oldstyle" support from nbd-server and nbd-client.
* Thu Mar  5 2015 mpluskal@suse.com
- Cleanup spec-file with spec-cleaner
- Do not generate config.example from README
  * add config.example as source
- Add unit file (nbd-server.service)
- Add sysconfig template (nbd-server.sysconfig)
- Add no-return-nonvoid.patch
- Update requirements
- Update to 3.9
  * AF_UNIX support
  * Fixed test suite on non-GNU getopt() implementations
  * New "treefiles" mode, which exports a gazillion of page-sized
    files; useful for exporting things which are stored on an
    SSHFS or amazon AWS (trough FUSE) or similar, where every
    write causes an upload to the backend storage
  * New "cowdir" option, allowing to specify where copy-on-write
    files are written.
  * Minor changes so that nbd-client can now also be compiled for
    the Android platform. This required removal of the -swap
    command line option, which requires the mlockall() system
    call, unavailable on Android.
  * Protocol update: a reserved bit is used to avoid sending
    the 124 bytes of useless data at the beginning of the
    negotiation. The change is implemented so that things will
    still work with clients not supporting this option, however.
  * gznbd is now built by the same build system, rather than
    a separate one. Note however that gznbd is still
    unmaintained; it should be considered a "contrib" feature.
  * "nbd-server -V" will now output the nbd-server version number.
  * Various fixes found through Coverity and the clang static
    analyzer, and lots of other minor things too small to
    mention here.
* Sun Nov  9 2014 Led <ledest@gmail.com>
- fix bashisms in pre script
* Thu Apr 18 2013 dmueller@suse.com
- update to 3.3:
  * and various bugfixes, upgrading is strongly recommended
  * Fix various nbd-server misbehave bugs
* Tue Jul 10 2012 aj@suse.de
- Fix restart_on_update usage.
* Fri May 18 2012 tabraham@novell.com
- Update to nbd-3.1
  * minor bug fixes
  * new feature: ability for the client to list the exports a server
    supports
* Wed Feb 29 2012 tabraham@novell.com
-  Update to nbd-3.0
  * minor bug fixes
- removed obsolete nbd-2.9.25-have_falloc_ph.diff
- compile with -fno-strict-aliasing to prevent breakage with new gcc
* Thu Jan  5 2012 tabraham@novell.com
-  Update to nbd-2.9.25
  * TRIM now uses FALLOC_FL_PUNCH_HOLE
  * minor bug fixes
- removed obsolete nbd-2.9.20-bigendian.diff
- refreshed nbd-2.9.20-close.diff
* Fri Sep 16 2011 rfno@fietze-home.de
- Install nbd-server.5 man page
* Mon Apr  4 2011 ms@suse.de
- socklen_t is always 32bit, size_t is 64bit. so a classical
  big endian 64bit error (bnc #674281)
* Fri Feb 25 2011 ms@suse.de
- Update to nbd-2.9.20:
  * Documentation update
  * Fix buffer size checking (bnc #674281)
  * Extend test suite and fix named exports
* Wed Aug  5 2009 ms@suse.de
- fixed missing close to open (bnc #528272)
* Thu Jul  9 2009 ms@suse.de
- Update to nbd-2.9.13:
  * Documentation update
  * Don't try working on partitions
    When a partition is given to the -d or -c command line options, reduce
    it to the master device instead. This assumes udev-style naming
    of device entries, though.
  * Fix autodetection
    Everything else in nbd-server was already 64bit-clean, but this part,
    where the size of a block device is autodected, wasn't.
    Replace the ioctl() to BLKGETSIZE with its 64bit version, so that trying
    to detect the size of a multi-terabyte block device does not fail.
  * Avoid unaligned access
  * Actually go daemon
    just doing chdir("/") leave file descriptors open, which is not ideal.
    Doing it this way closes everything, including any system-specific
    things we're not aware of.
  * Remove support for timeout option
* Mon May  4 2009 ms@suse.de
- Update to nbd-2.9.12:
  * use atoll instead of atol ... so that multi-terabyte exports work, too
  * Change meaning of -swap option: rather than trying to use a kernel patch
    which has been effectively dead since sometime around 2.1.something, do an
    mlockall() so that the kernel won't throw us out. The MCL_CURRENT | MCL_FUTURE
    bit might prove to be problematic, but if so, we'll deal with it when
    we get there
  * Use right printf() format string
* Wed Aug 20 2008 garloff@suse.de
- Update to nbd-2.9.11:
  * nbd-client: new option -c to check for connection, -d to
    drop connection, -persist to infinitely retry connections
    dropped by server. timeout= setting
  * nbd-server: drop autoreadonly option
* Thu Nov 15 2007 garloff@suse.de
- Update to nbd-2.9.9:
  * Document -C option of nbd-server.
  * Fix name of /var/run/ file.
* Wed Nov  7 2007 garloff@suse.de
- Split nbd into nbd and nbd-doc packages.
- Update to nbd-2.9.8:
  * nbd-server can now setuid to specified user and group.
  * prerun and postrun scripts can be specified.
  * IP address to bind to (listenaddress) can be configured.
  * Socket Direct Protocol (sdp) -- not enabled.
  * server exits now after connection has gone (oops!).
  * nbd-client: timeout parameter.
* Mon Jul 30 2007 garloff@suse.de
- Convert options from old nbd-server.conf to new nbd-server/config
  on update and drop support for old config file syntax.
- Fix daemonize(0) segfault and pid filename.
- Fix compiler warnings.
- Fix /dev filename docu.
- Use startproc, killproc, checkproc in init script
- nbd-server does not support SIGHUP, fix init script accordingly.
* Mon Jul 16 2007 garloff@suse.de
- Update to nbd-2.9.5:
  * Support for more platforms, pkgconfig, debug build.
  * -o option to convert cmdline options to config file.
  * Fix duplication in handling nonblocking server socket.
* Mon Mar 12 2007 garloff@suse.de
- Update to 2.9.2:
  * make child socket nonblocking
  * Reintegrate fix for 64bit on looking for file size.
  * Fix cross-compiling
  * Search for libraries to allow linking on Solaris
- Update to 2.9.1:
  * A fix for a piece of code following a null pointer, and one
    for a memory leak (if only minor).
- Update to 2.9.0:
  * * Configuration file support
  * The ability to server multiple nbd-server exports from one server
  * Better multiple-file support (files can now be of any random size)
  * Better support for virtualizing servers (in addition to replacing
    %%s with the client's IP address, it is now also possible to use
    other options which allow for hashed directories)
  * Support for changing uid and gid after startup
  * The ability to do the copy-on-write stuff with a sparse file rather
    than a mapped file (specified by a config file option), which may be
    slightly faster.
* Sat Nov 18 2006 garloff@suse.de
- Add nbd-server init script (parses /etc/nbd-server.conf).
* Thu Oct 19 2006 garloff@suse.de
- Update to nbd-2.8.7:
  * A fix for build against nbd.h for kernel 2.6.18 (and above,
    presumably)
* Thu Oct 12 2006 ro@suse.de
- include linux/types.h before linux/nbd.h
* Mon Aug  7 2006 garloff@suse.de
- Update to nbd-2.8.6:
  * performance/reliability improvements: Use select() rather than
    accept() and waitpid() with WNOHANG rather than wait().
- Update to nbd-2.8.5:
  * -persist option.
  * Add export verification.
  * Make sure queue is empty for write requests to read-only export.
- Update to nbd-2.8.4:
  * Includes the previous fix (closing server socket)
* Mon Mar  6 2006 garloff@suse.de
- Make sure children quit when they finish serving, rather than
  trying to do stuff with a server socket that they've closed
  already in a loop. Whoops. (Fix from nbd-2.8.4.)
* Wed Jan 25 2006 mls@suse.de
- converted neededforbuild to BuildRequires
* Mon Jan 23 2006 garloff@suse.de
- Enable -fstack-protector.
* Mon Jan  9 2006 garloff@suse.de
- Update to nbd-2.8.3:
  * CVE-2005-3534.
  * A fix for 64bit systems regarding the BLKGETSIZE ioctl().
  * Better handling of SIGCHLD.
* Mon Dec 19 2005 garloff@suse.de
- Update to nbd-2.8.2:
  * Enable LFS by default.
* Wed Nov  2 2005 dmueller@suse.de
- don't build as root
* Fri Oct 28 2005 garloff@suse.de
- Update to version 2.8.1.
- Documentation added.
* Mon Jul  4 2005 garloff@suse.de
- Initial creation of package.
